class WebhookAuthenticateException(Exception):
    pass


class AttachmentDecryptionException(Exception):
    pass


class MissingPublicKeyException(Exception):
    pass


class MissingSecretKeyException(Exception):
    pass
